@@ -1,11 +1,11 @@
 
-__version__ = "10.3.5"
+__version__ = "11.0.2"
 __author__ = "Alex Al-Saffar"
-__date__ = "2023-01-13"
+__date__ = "2023-03-17"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 pypi = r"https://pypi.org/project/crocodile"
 release_notes = """
-added ascci art for windows
+refurbished Cluster
 """
 
