@@ -3,7 +3,7 @@
 import getpass
 import platform
 import crocodile.toolbox as tb
-from crocodile.cluster.remote_machine import ResourceManager
+from crocodile.cluster.remote_machine import ResourceManager, WorkloadParams
 from importlib.machinery import SourceFileLoader
 from rich.console import Console
 from rich.panel import Panel
@@ -17,7 +17,7 @@ console = Console()
 
 # EXTRA-PLACEHOLDER-PRE
 
-_ = SourceFileLoader
+_ = SourceFileLoader, WorkloadParams
 
 to_be_deleted = ['res = ""  # to be overridden by execution line.', 'exec_obj = ""  # to be overridden by execution line.']
 # items below are defined to silence IDE warnings. They will be deleted by script preparer, then later defined by function to be executed.
@@ -29,29 +29,27 @@ rel_full_path = ""
 repo_path = ""
 func_name = ""
 func_module = ""
+func_class = ""
 
 description = ""
 ssh_repr = ""
 ssh_repr_remote = ""
 error_message = "No error message."  # to be updated by try-except block inside execution line.
-
-job_id = ""
-lock_resources = ""
 zellij_session = ""
-
+resource_manager_path = ""
 
 print("\n" * 2)
-manager = ResourceManager(job_id, platform.system())
-if lock_resources: manager.secure_resources()
+manager = ResourceManager.from_pickle(resource_manager_path)
+manager.secure_resources()
 
 # keep those values after lock is released
 time_at_execution_start_utc = pd.Timestamp.utcnow()
 time_at_execution_start_local = pd.Timestamp.now()
+manager.execution_log_dir.expanduser().create().joinpath("start_time.txt").write_text(str(time_at_execution_start_local))
 
 repo_path = tb.P(rf'{repo_path}').expanduser().absolute()
 tb.sys.path.insert(0, repo_path.str)
 func_kwargs = manager.kwargs_path.readit()
-manager.execution_log_dir.expanduser().create().joinpath("start_time.txt").write_text(str(time_at_execution_start_local))
 
 # EXTRA-PLACEHOLDER-POST
 
@@ -69,14 +67,15 @@ print("\n" * 2)
 if func_module is not None:
     # noinspection PyTypeChecker
     module = __import__(func_module, fromlist=[None])
-    exec_obj = module.__dict__[func_name]
+    exec_obj = module.__dict__[func_name] if not bool(func_class) else getattr(module.__dict__[func_class], func_name)
 elif func_name is not None:
     # This approach is not conducive to parallelization since "mymod" is not pickleable.
     module = SourceFileLoader("mymod", tb.P.home().joinpath(rel_full_path).as_posix()).load_module()  # loading the module assumes its not a script, there should be at least if __name__ == __main__ wrapping any script.
-    exec_obj = getattr(module, func_name)  # for README.md generation.
+    exec_obj = getattr(module, func_name) if not bool(func_class) else getattr(getattr(module, func_class), func_name)
 else:
     module = tb.P.home().joinpath(rel_full_path).readit()  # uses runpy to read .py files.
     exec_obj = module  # for README.md generation.
+# TODO: replace the above with a simple import statement that can be tested at local machine side.
 
 # execution_line
 
@@ -87,15 +86,19 @@ print("\n" * 2)
 # ######################### END OF EXECUTION #############################
 
 
-if lock_resources: manager.unlock_resources()
+manager.unlock_resources()
 
 if type(res) is tb.P or (type(res) is str and tb.P(res).expanduser().exists()):
     res_folder = tb.P(res).expanduser()
 else:
-    res_folder = tb.P.tmp(folder=rf"tmp_dirs/{job_id}").create()
+    res_folder = tb.P.tmp(folder=rf"tmp_dirs/{manager.job_id}").create()
     console.print(Panel(f"WARNING: The executed function did not return a path to a results directory. Execution metadata will be saved separately in {res_folder.collapseuser().as_posix()}."))
     print("\n\n")
-    tb.Save.pickle(obj=res, path=res_folder.joinpath("result.pkl"))
+    try:
+        tb.Save.pickle(obj=res, path=res_folder.joinpath("result.pkl"))
+    except TypeError as e:
+        print(e)
+        print(f"Could not pickle res object to path `{res_folder.joinpath('result.pkl').collapseuser().as_posix()}`.")
 
 time_at_execution_end_utc = pd.Timestamp.utcnow()
 time_at_execution_end_local = pd.Timestamp.now()
@@ -113,7 +116,7 @@ tb.Experimental.generate_readme(path=manager.root_dir.expanduser().joinpath("exe
 
 Job executed via tb.cluster.Machine
 remote: {ssh_repr}
-job_id: {job_id}
+job_id: {manager.job_id}
 
 py_script_path @ `{manager.py_script_path.collapseuser()}`
 shell_script_path @ `{manager.shell_script_path.collapseuser()}`
@@ -128,7 +131,7 @@ kwargs_path @ `{manager.kwargs_path.collapseuser()}`
 ''')
 
 
-manager.root_dir.expanduser().copy(folder=res_folder)
+manager.root_dir.expanduser().copy(folder=res_folder, overwrite=True)
 
 # print to execution console:
 exec_times.print(title="Execution Times", as_config=True)
@@ -145,5 +148,5 @@ if zellij_session != "":
     tb.Terminal().run(f"""zellij --session {zellij_session} action write-chars "cd {res_folder.as_posix()};lf" """)
 
 
-print(f"job {job_id} is completed.")
+print(f"job {manager.job_id} is completed.")
 # if lock_resources and interactive: print(f"This jos is interactive. Don't forget to close it as it is also locking resources.")
