@@ -5,12 +5,13 @@ import random
 import textwrap
 import subprocess
 from crocodile.core import install_n_import
-
+from pathlib import Path
 
 # https://github.com/VaasuDevanS/cowsay-python
 # https://github.com/pwaller/pyfiglet
 # https://github.com/tehmaze/lolcat/
 # https://github.com/wasi-master/gradient_figlet/
+# https://github.com/sepandhaghighi/art
 
 """
 
@@ -49,11 +50,11 @@ class CowStyles:
 
 FIGLET_FONTS = ['banner', 'big', 'standard']
 
-FIGJS_FONTS = ['3D Diagonal', '3D-ASCII', '4Max', '5 Line Oblique', 'Acrobatic', 'Alligator2',               'AMC Tubes', 'ANSI Regular', 'ANSI Shadow', 'Avatar', 'Banner', 'Banner3-D', 'Banner3', 'Banner4',
+FIGJS_FONTS = ['3D Diagonal', '3D-ASCII', '4Max', '5 Line Oblique', 'Acrobatic', 'ANSI Regular', 'ANSI Shadow', 'Avatar', 'Banner', 'Banner3-D', 'Banner4',
                'Basic', 'Big Money-ne', 'Big Money-nw', 'Big Money-se', 'Big Money-sw', 'Big', 'Bloody', 'Bolger', 'Braced', 'Bright',
                'DOS Rebel',
                'Elite', 'Epic', 'Flower Power',
-               'Fraktur', # 'Isometric4'
+               'Fraktur', # 'Isometric4'. 'AMC Tubes', 'Banner3', Alligator2
                'Star Wars',
                'Sub-Zero', 'The Edge', 'USA Flag', 'Varsity', "Doom"]  # too large  Crazy 'Sweet', 'Electronic', 'Swamp Land', Crawford, Alligator
 
@@ -70,15 +71,18 @@ def get_art(comment=None, artlib=None, style=None, super_style='scene', prefix='
         if style is None: style = random.choice(BoxStyles.__dict__[super_style or random.choice(['language', 'scene', 'character'])])
         fonting = f'figlet -f {random.choice(FIGLET_FONTS)}'
         cmd = f"""echo "{comment}" | {fonting} | boxes -d {style} {to_file}"""
+        # tmpfile = Path.home().joinpath("tmp_results/tmp_figlet.txt")
+        # tmpfile.write_text(ArtLib.figlet(comment))
+        # cmd = f"""cat {tmpfile} | boxes -d {style} {to_file}"""
     else:
         if style is None: style = random.choice(CowStyles.figures)
-        cmd = f"""echo "{comment}" | /usr/games/cowsay -f {style} {to_file}"""
+        cmd = f"""echo "{comment}" | cowsay -f {style} {to_file}"""
     res = subprocess.run(cmd, text=True, capture_output=True, shell=True).stdout
     res = textwrap.indent(res, prefix=prefix)
     if verbose:
         print(f'Using style: {style} from {artlib}', '\n' * 3)
-        print(cmd)
-        print(res)
+        print(f'{cmd=}')
+        print(f'Results:\n', res)
     return res
 
 
